img.tracvatar.ticket-reporter {
  float:left;
  padding:0 10px 10px 0px;
}

img.tracvatar.browser-lineitem {
  padding:0 3px 0 0;
}

img.tracvatar.browser-changeset {
  padding:3px 5px 0px 0px;
}

td.tracvatar.prefs-gravatar {
  vertical-align:middle;
  white-space:nowrap;
}